﻿using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;
using Primitives3D;

namespace Lab8
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        TeapotPrimitive teapot;
        Effect effect;

        SpherePrimitive pointLight;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here
            teapot = new TeapotPrimitive(GraphicsDevice);
            pointLight = new SpherePrimitive(GraphicsDevice, 0.1f, 20);
            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            // TODO: use this.Content to load your game content here
            effect = Content.Load<Effect>("light");
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            // TODO: Add your update logic here

            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.Black);

            // TODO: Add your drawing code here
            Vector3 cameraPosition = new Vector3(0, 0, 8);
            Matrix world = Matrix.CreateRotationY((float)gameTime.TotalGameTime.TotalSeconds);
            Matrix view = Matrix.CreateLookAt(cameraPosition, Vector3.Zero, Vector3.Up);
            Matrix proj = Matrix.CreatePerspectiveFieldOfView(MathHelper.ToRadians(45), GraphicsDevice.Viewport.AspectRatio, 0.1f, 20);


            Vector3 lightPos0 = new Vector3(0, -1, 1);
            Vector3 lightPos1 = new Vector3(1, 1, 1);
            Vector3 lightPos2 = new Vector3(-2, 0, 0);
            // Источники света

            pointLight.Draw(Matrix.CreateTranslation(lightPos0), view, proj, Color.Blue);
            pointLight.Draw(Matrix.CreateTranslation(lightPos1), view, proj, Color.Blue);
            pointLight.Draw(Matrix.CreateTranslation(lightPos2), view, proj, Color.Blue);


            effect.CurrentTechnique = effect.Techniques["Phong"];

            effect.Parameters["LightPosition0"].SetValue(lightPos0);
            effect.Parameters["LightPosition1"].SetValue(lightPos1);
            effect.Parameters["LightPosition2"].SetValue(lightPos2);

            effect.Parameters["AmbientColor0"].SetValue(new Vector4(0.1f, 0.1f, 0.1f, 1));
            effect.Parameters["DiffuseColor0"].SetValue(new Vector4(0.5f, 0f, 0f, 1));
            effect.Parameters["SpecularColor0"].SetValue(new Vector4(1, 0, 0, 1));

            effect.Parameters["AmbientColor1"].SetValue(new Vector4(0.1f, 0.1f, 0.1f, 1));
            effect.Parameters["DiffuseColor1"].SetValue(new Vector4(0, 0.5f, 0.5f, 1));
            effect.Parameters["SpecularColor1"].SetValue(new Vector4(0, 1, 1, 1));

            effect.Parameters["AmbientColor2"].SetValue(new Vector4(0.1f, 0.1f, 0.1f, 1));
            effect.Parameters["DiffuseColor2"].SetValue(new Vector4(0.5f, 0.5f, 0.5f, 1));
            effect.Parameters["SpecularColor2"].SetValue(new Vector4(1, 1, 1, 1));

            effect.Parameters["Eye"].SetValue(cameraPosition);
            effect.Parameters["World"].SetValue(world);
            effect.Parameters["View"].SetValue(view);
            effect.Parameters["Projection"].SetValue(proj);

            effect.Parameters["Color"].SetValue(new Vector4(1, 1, 0, 1));

            teapot.Draw(effect);

            effect.Parameters["World"].SetValue(world * Matrix.CreateTranslation(0, 2, 0));
            teapot.Draw(effect);

            effect.Parameters["World"].SetValue(world * Matrix.CreateTranslation(-3, 0, 0));
            teapot.Draw(effect);

            base.Draw(gameTime);
        }
    }
}
